@@ -74,6 +74,7 @@ class ForagingEnv(Env):
         sight,
         max_episode_steps,
         force_coop,
+        normalize_reward=True,
     ):
         self.logger = logging.getLogger(__name__)
         self.seed()
@@ -95,6 +96,10 @@ class ForagingEnv(Env):
         self._valid_actions = None
         self._max_episode_steps = max_episode_steps
 
+        self._normalize_reward = normalize_reward
+
+        self.viewer = None
+
     def seed(self, seed=None):
         self.np_random, seed = seeding.np_random(seed)
         return [seed]
@@ -118,7 +123,7 @@ class ForagingEnv(Env):
             self.max_player_level,
         ] * len(self.players)
 
-        return gym.spaces.Box(np.array(min_obs), np.array(max_obs), dtype=np.int32)
+        return gym.spaces.Box(np.array(min_obs), np.array(max_obs), dtype=np.float32)
 
     @classmethod
     def from_obs(cls, obs):
@@ -210,7 +215,7 @@ class ForagingEnv(Env):
 
         food_count = 0
         attempts = 0
-        min_level = max([p.level for p in self.players]) + 1 if self.force_coop else 1
+        min_level = max_level if self.force_coop else 1
 
         while food_count < max_food and attempts < 1000:
             attempts += 1
@@ -314,12 +319,20 @@ class ForagingEnv(Env):
                     reward=a.reward if a == player else None,
                 )
                 for a in self.players
-                if min(
+                if (
+                    min(
+                        self._transform_to_neighborhood(
+                            player.position, self.sight, a.position
+                        )
+                    )
+                    >= 0
+                )
+                and max(
                     self._transform_to_neighborhood(
                         player.position, self.sight, a.position
                     )
                 )
-                >= 0
+                <= 2 * self.sight
             ],
             # todo also check max?
             field=np.copy(self.neighborhood(*player.position, self.sight)),
@@ -330,8 +343,13 @@ class ForagingEnv(Env):
 
     def _make_gym_obs(self, observations):
         def make_obs_array(observation):
-            obs = np.zeros(self.observation_space[0].shape)
+            obs = np.zeros(self.observation_space[0].shape, dtype=np.float32)
             # obs[: observation.field.size] = observation.field.flatten()
+            # self player is always first
+            seen_players = [p for p in observation.players if p.is_self] + [
+                p for p in observation.players if not p.is_self
+            ]
+
             for i in range(self.max_food):
                 obs[3 * i] = -1
                 obs[3 * i + 1] = -1
@@ -342,7 +360,12 @@ class ForagingEnv(Env):
                 obs[3 * i + 1] = x
                 obs[3 * i + 2] = observation.field[y, x]
 
-            for i, p in enumerate(observation.players):
+            for i in range(len(self.players)):
+                obs[self.max_food * 3 + 3 * i] = -1
+                obs[self.max_food * 3 + 3 * i + 1] = -1
+                obs[self.max_food * 3 + 3 * i + 2] = 0
+
+            for i, p in enumerate(seen_players):
                 obs[self.max_food * 3 + 3 * i] = p.position[0]
                 obs[self.max_food * 3 + 3 * i + 1] = p.position[1]
                 obs[self.max_food * 3 + 3 * i + 2] = p.level
@@ -448,9 +471,11 @@ class ForagingEnv(Env):
 
             # else the food was loaded and each player scores points
             for a in adj_players:
-                a.reward = float(a.level * food) / float(
-                    adj_player_level * self._food_spawned
-                )  # normalize reward
+                a.reward = float(a.level * food)
+                if self._normalize_reward:
+                    a.reward = a.reward / float(
+                        adj_player_level * self._food_spawned
+                    )  # normalize reward
             # and the food is removed
             self.field[frow, fcol] = 0
 
@@ -469,9 +494,14 @@ class ForagingEnv(Env):
         from .rendering import Viewer
 
         self.viewer = Viewer((self.rows, self.cols))
+        self._rendering_initialized = True
 
     def render(self, mode="human"):
         if not self._rendering_initialized:
             self._init_render()
 
-        self.viewer.render(self)
+        return self.viewer.render(self, return_rgb_array=mode == "rgb_array")
+
+    def close(self):
+        if self.viewer:
+            self.viewer.close()
